package net.knarcraft.ffmpegconverter.streams;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.HashMap;
import java.util.Map;

/**
 * A class for representing stream tags that might be found for streams in video files
 */
public enum StreamTag {

    /**
     * The absolute index of this stream in the file
     *
     * <p>Applicable for all 3 stream types</p>
     */
    INDEX("index"),
    /**
     * The name of the codec, useful for identification
     *
     * <p>Applicable for all 3 stream types</p>
     */
    CODEC_NAME("codec_name"),

    /**
     * The long name of the codec, useful for displaying information
     *
     * <p>Applicable for all 3 stream types</p>
     */
    CODEC_LONG_NAME("codec_long_name"),

    /**
     * The profile the encoder is set to
     *
     * <p>Applicable for all 3 stream types</p>
     */
    PROFILE("profile"),

    /**
     * Whether the type of codec for the stream is audio, video or subtitle
     *
     * <p>Applicable for all 3 stream types</p>
     */
    CODEC_TYPE("codec_type"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    CODEC_TAG_STRING("codec_tag_string"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    CODEC_TAG("codec_tag"),

    /**
     * <p>Applicable for audio streams</p>
     */
    SAMPLE_FMT("sample_fmt"),

    /**
     * <p>Applicable for audio streams</p>
     */
    SAMPLE_RATE("sample_rate"),

    /**
     * The number of channels in an audio stream
     *
     * <p>Applicable for audio streams</p>
     */
    CHANNELS("channels"),

    /**
     * Human-recognizable term for the number of audio channels, such as stereo, mono or surround
     *
     * <p>Applicable for audio streams</p>
     */
    CHANNEL_LAYOUT("channel_layout"),

    /**
     * <p>Applicable for audio streams</p>
     */
    BITS_PER_SAMPLE("bits_per_sample"),

    /**
     * <p>Applicable for audio streams</p>
     */
    INITIAL_PADDING("initial_padding"),

    /**
     * The viewable video width
     *
     * <p>Applicable for video and subtitle streams</p>
     */
    WIDTH("width"),

    /**
     * The viewable video height
     *
     * <p>Applicable for video and subtitle streams</p>
     */
    HEIGHT("height"),

    /**
     * The original video width, before any padding was applied to account for resolution multiples
     *
     * <p>Applicable for video streams</p>
     */
    CODED_WIDTH("coded_width"),

    /**
     * The original video height, before any padding was applied to account for resolution multiples
     *
     * <p>Applicable for video streams</p>
     */
    CODED_HEIGHT("coded_height"),

    /**
     * <p>Applicable for video streams</p>
     */
    CLOSED_CAPTIONS("closed_captions"),

    /**
     * <p>Applicable for video streams</p>
     */
    FILM_GRAIN("film_grain"),

    /**
     * <p>Applicable for video streams</p>
     */
    HAS_B_FRAMES("has_b_frames"),

    /**
     * The aspect ratio used to stretch the video for playback
     *
     * <p>Applicable for video streams</p>
     */
    SAMPLE_ASPECT_RATIO("sample_aspect_ratio"),

    /**
     * The aspect ratio of the video stream
     *
     * <p>Applicable for video streams</p>
     */
    DISPLAY_ASPECT_RATIO("display_aspect_ratio"),

    /**
     * The pixel format used for the video stream
     *
     * <p>Applicable for video streams</p>
     */
    PIX_FMT("pix_fmt"),

    /**
     * The quality level of the video stream
     *
     * <p>Applicable for video streams</p>
     */
    LEVEL("level"),

    /**
     * <p>Applicable for video streams</p>
     */
    COLOR_RANGE("color_range"),

    /**
     * How colors are stored in the video stream's file
     *
     * <p>Applicable for video streams</p>
     */
    COLOR_SPACE("color_space"),

    /**
     * <p>Applicable for video streams</p>
     */
    COLOR_TRANSFER("color_transfer"),

    /**
     * <p>Applicable for video streams</p>
     */
    COLOR_PRIMARIES("color_primaries"),

    /**
     * <p>Applicable for video streams</p>
     */
    CHROMA_LOCATION("chroma_location"),

    /**
     * <p>Applicable for video streams</p>
     */
    FIELD_ORDER("field_order"),

    /**
     * <p>Applicable for video streams</p>
     */
    REFS("refs"),

    /**
     * <p>Applicable for video streams</p>
     */
    IS_AVC("is_avc"),

    /**
     * <p>Applicable for video streams</p>
     */
    NAL_LENGTH_SIZE("nal_length_size"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    ID("id"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    R_FRAME_RATE("r_frame_rate"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    AVERAGE_FRAME_RATE("avg_frame_rate"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TIME_BASE("time_base"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    START_PTS("start_pts"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    START_TIME("start_time"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DURATION_TS("duration_ts"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DURATION("duration"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    BIT_RATE("bit_rate"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    MAX_BIT_RATE("max_bit_rate"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    BITS_PER_RAW_SAMPLE("bits_per_raw_sample"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    NB_FRAMES("nb_frames"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    NB_READ_FRAMES("nb_read_frames"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    NB_READ_PACKETS("nb_read_packets"),

    /**
     * <p>Applicable for video and subtitle streams</p>
     */
    EXTRA_DATA_SIZE("extradata_size"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_DEFAULT("DISPOSITION:default"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_DUB("DISPOSITION:dub"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_ORIGINAL("DISPOSITION:original"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_COMMENT("DISPOSITION:comment"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_LYRICS("DISPOSITION:lyrics"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_KARAOKE("DISPOSITION:karaoke"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_FORCED("DISPOSITION:forced"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_HEARING_IMPAIRED("DISPOSITION:hearing_impaired"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_VISUAL_IMPAIRED("DISPOSITION:visual_impaired"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_CLEAN_EFFECTS("DISPOSITION:clean_effects"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_ATTACHED_PIC("DISPOSITION:attached_pic"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_TIMED_THUMBNAILS("DISPOSITION:timed_thumbnails"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_NON_DIEGETIC("DISPOSITION:non_diegetic"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_CAPTIONS("DISPOSITION:captions"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_DESCRIPTIONS("DISPOSITION:descriptions"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_METADATA("DISPOSITION:metadata"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_DEPENDENT("DISPOSITION:dependent"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    DISPOSITION_STILL_IMAGE("DISPOSITION:still_image"),

    /**
     * The language of the stream
     *
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_LANGUAGE("TAG:language"),

    /**
     * The title of an audio stream
     *
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_TITLE("TAG:title"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_BPS_ENG("TAG:BPS-eng"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_DURATION_ENG("TAG:DURATION-eng"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_NUMBER_OF_FRAMES_ENG("TAG:NUMBER_OF_FRAMES-eng"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_NUMBER_OF_BYTES_ENG("TAG:NUMBER_OF_BYTES-eng"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_SOURCE_ID_ENG("TAG:SOURCE_ID-eng"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_SOURCE_ID("TAG:SOURCE_ID"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_STATISTICS_WRITING_APP_ENG("TAG:_STATISTICS_WRITING_APP-eng"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_STATISTICS_WRITING_APP("TAG:_STATISTICS_WRITING_APP"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_STATISTICS_WRITING_DATE_UTC_ENG("TAG:_STATISTICS_WRITING_DATE_UTC-eng"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_STATISTICS_WRITING_DATE_UTC("TAG:_STATISTICS_WRITING_DATE_UTC"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_STATISTICS_TAGS_ENG("TAG:_STATISTICS_TAGS-eng"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_STATISTICS_TAGS("TAG:_STATISTICS_TAGS"),

    /**
     * <p>Applicable for video streams</p>
     */
    TAG_ENCODER("TAG:ENCODER"),

    /**
     * <p>Applicable for all 3 stream types</p>
     */
    TAG_DURATION("TAG:DURATION"),
    ;

    private static final Map<String, StreamTag> tagLookup = new HashMap<>();

    private final @NotNull String tagString;

    /**
     * Instantiates a new stream tag
     *
     * @param tagString <p>The tag string ffmpeg prints to specify this tag</p>
     */
    StreamTag(@NotNull String tagString) {
        this.tagString = tagString;
    }

    /**
     * Gets the stream tag defined by the given string
     *
     * @param input <p>The input string to parse</p>
     * @return <p>The corresponding stream tab, or null if not found</p>
     */
    @Nullable
    public static StreamTag getFromString(@NotNull String input) {
        if (tagLookup.isEmpty()) {
            for (StreamTag tag : StreamTag.values()) {
                tagLookup.put(tag.tagString, tag);
            }
        }
        return tagLookup.get(input);
    }

    @Override
    public String toString() {
        return this.tagString;
    }

}
