package net.knarcraft.ffmpegconverter.streams;

import org.jetbrains.annotations.NotNull;

import java.util.Map;

/**
 * An object representation of a subtitle stream in a media file
 */
public class SubtitleStream extends AbstractStream implements StreamObject {

    final private boolean isFullSubtitle;
    final private boolean isImageSubtitle;

    /**
     * Instantiates a new subtitle stream
     *
     * @param streamInfo    <p>All info about the stream</p>
     * @param inputIndex    <p>The index of the input file containing this stream</p>
     * @param relativeIndex <p>The index of the subtitle stream relative to other subtitle streams.</p>
     */
    public SubtitleStream(@NotNull Map<StreamTag, String> streamInfo, int inputIndex, int relativeIndex) {
        super(streamInfo, inputIndex, relativeIndex);
        this.isFullSubtitle = isFullSubtitle();
        this.isImageSubtitle = codecName != null &&
                (getCodecName().equals("hdmv_pgs_subtitle") || getCodecName().equals("dvd_subtitle"));
    }

    /**
     * Gets whether the subtitle is an image subtitle
     *
     * @return <p>Whether the subtitles is an image subtitle.</p>
     */
    public boolean getIsImageSubtitle() {
        return this.isImageSubtitle;
    }

    /**
     * Gets whether the subtitle is a full subtitle
     *
     * @return <p>Whether the subtitle is a full subtitle.</p>
     */
    public boolean getIsFullSubtitle() {
        return this.isFullSubtitle;
    }

    /**
     * Checks whether the subtitle translates everything (as opposed to just songs and signs)
     *
     * @return <p>True if the subtitle translates everything.</p>
     */
    private boolean isFullSubtitle() {
        String titleLowercase = getTitle().toLowerCase();
        return !titleLowercase.matches(".*si(ng|gn)s?[ &/a-z]+songs?.*") &&
                !titleLowercase.matches(".*songs?[ &/a-z]+si(gn|ng)s?.*") &&
                !titleLowercase.matches(".*forced.*") &&
                !titleLowercase.matches(".*s&s.*") &&
                !titleLowercase.matches("signs?") &&
                !titleLowercase.matches("songs?") &&
                !titleLowercase.matches(".*signs only.*") &&
                !titleLowercase.matches(".* signs .*") &&
                !titleLowercase.matches("signs@.*");
    }

    @Override
    public char streamTypeCharacter() {
        return 's';
    }

}