package net.knarcraft.ffmpegconverter.utility;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.File;
import java.util.List;

/**
 * A class which helps with file handling
 */
public final class FileUtil {

    private FileUtil() {

    }

    /**
     * Gets the path described by the input, but changed to account for collisions
     *
     * @param folder       <p>The folder containing the output file.</p>
     * @param file         <p>The input file.</p>
     * @param outExtension <p>The extension of the output file.</p>
     * @return <p>A file name with the new extension and without any collisions.</p>
     */
    @NotNull
    public static String getNonCollidingPath(@NotNull File folder, @NotNull File file, @NotNull String outExtension) {
        return FileUtil.getNonCollidingFilename(folder.getAbsolutePath() + File.separator +
                FileUtil.stripExtension(file.getName()) + "." + outExtension, outExtension);
    }

    /**
     * Recursively lists all files in a folder
     *
     * @param folder        <p>The folder to start from.</p>
     * @param maxRecursions <p>Maximum number of recursions</p>
     * @return A list of files
     */
    @Nullable
    public static File[] listFilesRecursive(@NotNull File folder, @NotNull List<String> extensions, int maxRecursions) {
        //Return if the target depth has been reached
        if (maxRecursions == 0) {
            return null;
        }
        //Get a list of all files which are folders and has one of the extensions specified
        File[] foundFiles = folder.listFiles((file) -> file.isFile() &&
                ListUtil.listContains(extensions, (item) -> file.getName().toLowerCase().endsWith(item)));

        //Return if recursion is finished
        if (maxRecursions == 1) {
            return foundFiles;
        }
        //Get all folders in the directory
        File[] subFolders = folder.listFiles((dir, name) -> new File(dir, name).isDirectory());
        //Return if the folder has no sub folders
        if (subFolders == null) {
            return foundFiles;
        }
        for (File subFolder : subFolders) {
            //Get all relevant files contained within the sub folder
            File[] nextLevel = listFilesRecursive(subFolder, extensions, maxRecursions - 1);
            //Add found files to the output
            if (nextLevel != null) {
                if (foundFiles == null) {
                    foundFiles = nextLevel;
                } else {
                    foundFiles = ListUtil.concatenate(foundFiles, nextLevel);
                }
            }
        }
        return foundFiles;
    }

    /**
     * Adds parentheses with an integer if the output file already exists
     *
     * @param targetPath <p>The path the file should ideally be saved at.</p>
     * @param extension  <p>The extension of the target file.</p>
     * @return <p>A filename guaranteed not to collide with other files.</p>
     */
    @NotNull
    private static String getNonCollidingFilename(@NotNull String targetPath, @NotNull String extension) {
        File newFile = new File(targetPath);
        String fileName = stripExtension(targetPath).replaceAll("\\([0-9]+\\)$", "");
        int i = 1;
        while (newFile.exists()) {
            newFile = new File(fileName + "(" + i++ + ")" + "." + extension);
        }
        return newFile.toString();
    }

    /**
     * Gets the extension of the given filename
     *
     * @param file <p>The filename to check</p>
     * @return <p>The file's extension</p>
     */
    @NotNull
    public static String getExtension(@NotNull String file) {
        if (file.contains(".")) {
            return file.substring(file.lastIndexOf('.') + 1);
        } else {
            return "";
        }
    }

    /**
     * Removes the extension from a file name
     *
     * @param file <p>A filename.</p>
     * @return <p>A filename without its extension.</p>
     */
    @NotNull
    public static String stripExtension(@NotNull String file) {
        return file.substring(0, file.lastIndexOf('.'));
    }

}
